@@ -1,15 +1,27 @@
-/* Copyright (C) 2001, 2002, 2003, 2004, 2005 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.
+   This file is part of Red Hat elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2001.
 
-   This program is Open Source software; you can redistribute it and/or
-   modify it under the terms of the Open Software License version 1.0 as
-   published by the Open Source Initiative.
+   Red Hat elfutils is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by the
+   Free Software Foundation; version 2 of the License.
 
-   You should have received a copy of the Open Software License along
-   with this program; if not, you may obtain a copy of the Open Software
-   License version 1.0 from http://www.opensource.org/licenses/osl.php or
-   by writing the Open Source Initiative c/o Lawrence Rosen, Esq.,
-   3001 King Ranch Road, Ukiah, CA 95482.   */
+   Red Hat elfutils is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public License along
+   with Red Hat elfutils; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301 USA.
+
+   Red Hat elfutils is an included package of the Open Invention Network.
+   An included package of the Open Invention Network is a package for which
+   Open Invention Network licensees cross-license their patents.  No patent
+   license is granted, either expressly or impliedly, by designation as an
+   included package.  Should you wish to participate in the Open Invention
+   Network licensing program, please visit www.openinventionnetwork.com
+   <http://www.openinventionnetwork.com>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -50,7 +62,6 @@ enum
     ARGP_static,
     ARGP_dynamic,
     ARGP_pagesize,
-    ARGP_rpath,
     ARGP_rpath_link,
     ARGP_runpath,
     ARGP_runpath_link,
@@ -59,6 +70,9 @@ enum
     ARGP_no_gc_sections,
     ARGP_no_undefined,
     ARGP_conserve,
+    ARGP_as_needed,
+    ARGP_no_as_needed,
+    ARGP_eh_frame_hdr,
 #if YYDEBUG
     ARGP_yydebug,
 #endif
@@ -68,23 +82,26 @@ enum
 /* Definitions of arguments for argp functions.  */
 static const struct argp_option options[] =
 {
-  /* XXX This list will be reordered and section names will be added.
-     Just not right now.  */
+  { NULL, 0, NULL, 0, N_("Input File Control:"), 0 },
   { "whole-archive", ARGP_whole_archive, NULL, 0,
     N_("Include whole archives in the output from now on."), 0 },
   { "no-whole-archive", ARGP_no_whole_archive, NULL, 0,
     N_("Stop including the whole arhives in the output."), 0 },
+  { NULL, 'l', N_("FILE"), OPTION_HIDDEN, NULL, 0 },
+  { NULL, '(', NULL, 0, N_("Start a group."), 0 },
+  { NULL, ')', NULL, 0, N_("End a group."), 0 },
+  { NULL, 'L', N_("PATH"), 0,
+    N_("Add PATH to list of directories files are searched in."), 0 },
+  { "as-needed", ARGP_as_needed, NULL, 0,
+    N_("Only set DT_NEEDED for following dynamic libs if actually used"), 0 },
+  { "no-as-needed", ARGP_no_as_needed, NULL, 0,
+    N_("Always set DT_NEEDED for following dynamic libs"), 0 },
+  { "rpath-link", ARGP_rpath_link, "PATH", OPTION_HIDDEN, NULL, 0 },
+  { NULL, 'i', NULL, 0, N_("Ignore LD_LIBRARY_PATH environment variable."),
+    0 },
 
+  { NULL, 0, NULL, 0, N_("Output File Control:"), 0 },
   { "output", 'o', N_("FILE"), 0, N_("Place output in FILE."), 0 },
-
-  { NULL, 'O', N_("LEVEL"), OPTION_ARG_OPTIONAL,
-    N_("Set optimization level to LEVEL."), 0 },
-
-  { "verbose", 'v', NULL, 0, N_("Verbose messages."), 0 },
-  { "trace", 't', NULL, 0, N_("Trace file opens."), 0 },
-  { "conserve-memory", ARGP_conserve, NULL, 0,
-    N_("Trade speed for less memory usage"), 0 },
-
   { NULL, 'z', "KEYWORD", OPTION_HIDDEN, NULL, 0 },
   { "-z nodefaultlib", '\0', NULL, OPTION_DOC,
     N_("Object is marked to not use default search path at runtime."), 0 },
@@ -116,74 +133,56 @@ Default rules of extracting from archive; weak references are not enough."),
     N_("Ignore/record dependencies on unused DSOs."), 0 },
   { "-z systemlibrary", '\0', NULL, OPTION_DOC,
     N_("Generated DSO will be a system library."), 0 },
-
-  { NULL, 'l', N_("FILE"), OPTION_HIDDEN, NULL, 0 },
-
-  { NULL, '(', NULL, 0, N_("Start a group."), 0 },
-  { NULL, ')', NULL, 0, N_("End a group."), 0 },
-
-  { NULL, 'L', N_("PATH"), 0,
-    N_("Add PATH to list of directories files are searched in."), 0 },
-
-  { NULL, 'c', N_("FILE"), 0, N_("Use linker script in FILE."), 0 },
-
   { "entry", 'e', N_("ADDRESS"), 0, N_("Set entry point address."), 0 },
-
   { "static", ARGP_static, NULL, OPTION_HIDDEN, NULL, 0 },
   { "-B static", ARGP_static, NULL, OPTION_DOC,
     N_("Do not link against shared libraries."), 0 },
   { "dynamic", ARGP_dynamic, NULL, OPTION_HIDDEN, NULL, 0 },
   { "-B dynamic", ARGP_dynamic, NULL, OPTION_DOC,
     N_("Prefer linking against shared libraries."), 0 },
-
   { "export-dynamic", 'E', NULL, 0, N_("Export all dynamic symbols."), 0 },
-
   { "strip-all", 's', NULL, 0, N_("Strip all symbols."), 0 },
   { "strip-debug", 'S', NULL, 0, N_("Strip debugging symbols."), 0 },
-
   { "pagesize", ARGP_pagesize, "SIZE", 0,
     N_("Assume pagesize for the target system to be SIZE."), 0 },
-
-  { "rpath", ARGP_rpath, "PATH", OPTION_HIDDEN, NULL, 0 },
-  { "rpath-link", ARGP_rpath_link, "PATH", OPTION_HIDDEN, NULL, 0 },
-
+  { "rpath", 'R', "PATH", OPTION_HIDDEN, NULL, 0 },
   { "runpath", ARGP_runpath, "PATH", 0, N_("Set runtime DSO search path."),
     0 },
   { "runpath-link", ARGP_runpath_link, "PATH", 0,
     N_("Set link time DSO search path."), 0 },
-
-  { NULL, 'i', NULL, 0, N_("Ignore LD_LIBRARY_PATH environment variable."),
-    0 },
-
-  { "version-script", ARGP_version_script, "FILE", 0,
-    N_("Read version information from FILE."), 0 },
-
-  { "emulation", 'm', "NAME", 0, N_("Set emulation to NAME."), 0 },
-
   { "shared", 'G', NULL, 0, N_("Generate dynamic shared object."), 0 },
   { NULL, 'r', NULL, 0L, N_("Generate relocatable object."), 0 },
-
   { NULL, 'B', "KEYWORD", OPTION_HIDDEN, "", 0 },
   { "-B local", 'B', NULL, OPTION_DOC,
     N_("Causes symbol not assigned to a version be reduced to local."), 0 },
-
   { "gc-sections", ARGP_gc_sections, NULL, 0, N_("Remove unused sections."),
     0 },
   { "no-gc-sections", ARGP_no_gc_sections, NULL, 0,
     N_("Don't remove unused sections."), 0 },
-
   { "soname", 'h', "NAME", 0, N_("Set soname of shared object."), 0 },
   { "dynamic-linker", 'I', "NAME", 0, N_("Set the dynamic linker name."), 0 },
-
   { NULL, 'Q', "YN", OPTION_HIDDEN, NULL, 0 },
   { "-Q y | n", 'Q', NULL, OPTION_DOC,
     N_("Add/suppress addition indentifying link-editor to .comment section"),
     0 },
+  { "eh-frame-hdr", ARGP_eh_frame_hdr, NULL, 0,
+    N_("Create .eh_frame_hdr section"), 0 },
 
+  { NULL, 0, NULL, 0, N_("Linker Operation Control:"), 0 },
+  { "verbose", 'v', NULL, 0, N_("Verbose messages."), 0 },
+  { "trace", 't', NULL, 0, N_("Trace file opens."), 0 },
+  { "conserve-memory", ARGP_conserve, NULL, 0,
+    N_("Trade speed for less memory usage"), 0 },
+  { NULL, 'O', N_("LEVEL"), OPTION_ARG_OPTIONAL,
+    N_("Set optimization level to LEVEL."), 0 },
+  { NULL, 'c', N_("FILE"), 0, N_("Use linker script in FILE."), 0 },
 #if YYDEBUG
   { "yydebug", ARGP_yydebug, NULL, 0,
     N_("Select to get parser debug information"), 0 },
 #endif
+  { "version-script", ARGP_version_script, "FILE", 0,
+    N_("Read version information from FILE."), 0 },
+  { "emulation", 'm', "NAME", 0, N_("Set emulation to NAME."), 0 },
 
   { NULL, 0, NULL, 0, NULL, 0 }
 };
@@ -339,7 +338,7 @@ main (int argc, char *argv[])
      statements in the script.  This simply must not happen.  */
   ldin = fopen (linker_script, "r");
   if (ldin == NULL)
-    error (EXIT_FAILURE, errno, gettext ("cannot open linker script \"%s\""),
+    error (EXIT_FAILURE, errno, gettext ("cannot open linker script '%s'"),
 	   linker_script);
   /* No need for locking.  */
   __fsetlocking (ldin, FSETLOCKING_BYCALLER);
@@ -579,7 +578,7 @@ parse_opt_1st (int key, char *arg,
 	    else
 	      {
 		error (0, 0,
-		       gettext ("invalid page size value \"%s\": ignored"),
+		       gettext ("invalid page size value '%s': ignored"),
 		       arg);
 		ld_state.pagesize = 0;
 	      }
@@ -587,7 +586,7 @@ parse_opt_1st (int key, char *arg,
       }
       break;
 
-    case ARGP_rpath:
+    case 'R':
       add_rxxpath (&ld_state.rpath, arg);
       break;
 
@@ -608,6 +607,10 @@ parse_opt_1st (int key, char *arg,
       ld_state.gc_sections = key == ARGP_gc_sections;
       break;
 
+    case ARGP_eh_frame_hdr:
+      ld_state.eh_frame_hdr = true;
+      break;
+
     case 's':
       if (arg == NULL)
 	{
@@ -624,6 +627,8 @@ parse_opt_1st (int key, char *arg,
     case 'O':
     case ARGP_whole_archive:
     case ARGP_no_whole_archive:
+    case ARGP_as_needed:
+    case ARGP_no_as_needed:
     case 'L':
     case '(':
     case ')':
@@ -720,6 +725,13 @@ parse_opt_2nd (int key, char *arg,
       ld_state.extract_rule = defaultextract;
       break;
 
+    case ARGP_as_needed:
+      ld_state.as_needed = true;
+      break;
+    case ARGP_no_as_needed:
+      ld_state.as_needed = false;
+      break;
+
     case ARGP_static:
     case ARGP_dynamic:
       /* Enable/disable use for DSOs.  */
@@ -858,7 +870,7 @@ print_version (FILE *stream, struct argp_state *state __attribute__ ((unused)))
 Copyright (C) %s Red Hat, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2005");
+"), "2006");
   fprintf (stream, gettext ("Written by %s.\n"), "Ulrich Drepper");
 }
 
@@ -897,17 +909,19 @@ parse_z_option (const char *arg)
 	   /* This is only meaningful if we create a DSO.  */
 	   && ld_state.file_type == dso_file_type)
     ld_state.dt_flags_1 |= DF_1_NOOPEN;
-  else if (strcmp (arg, "ignore") == 0)
-    ld_state.ignore_unused_dsos = true;
-  else if (strcmp (arg, "record") == 0)
-    ld_state.ignore_unused_dsos = false;
   else if (strcmp (arg, "systemlibrary") == 0)
     ld_state.is_system_library = true;
+  else if (strcmp (arg, "execstack") == 0)
+    ld_state.execstack = execstack_true;
+  else if (strcmp (arg, "noexecstack") == 0)
+    ld_state.execstack = execstack_false_force;
   else if (strcmp (arg, "allextract") != 0
 	   && strcmp (arg, "defaultextract") != 0
 	   && strcmp (arg, "weakextract") != 0
 	   && strcmp (arg, "lazyload") != 0
-	   && strcmp (arg, "nolazyload") != 0)
+	   && strcmp (arg, "nolazyload") != 0
+	   && strcmp (arg, "ignore") != 0
+	   && strcmp (arg, "record") != 0)
     error (0, 0, gettext ("unknown option `-%c %s'"), 'z', arg);
 }
 
@@ -925,6 +939,10 @@ parse_z_option_2 (const char *arg)
     ld_state.lazyload = true;
   else if (strcmp (arg, "nolazyload") == 0)
     ld_state.lazyload = false;
+  else if (strcmp (arg, "ignore") == 0)
+    ld_state.as_needed = true;
+  else if (strcmp (arg, "record") == 0)
+    ld_state.as_needed = false;
 }
 
 
@@ -1095,6 +1113,7 @@ ld_new_inputfile (const char *fname, enum file_type type)
   newfile->soname = newfile->fname = newfile->rfname = fname;
   newfile->file_type = type;
   newfile->extract_rule = ld_state.extract_rule;
+  newfile->as_needed = ld_state.as_needed;
   newfile->lazyload = ld_state.lazyload;
   newfile->status = not_opened;
 
@@ -1335,7 +1354,7 @@ read_version_script (const char *fname)
      or absolute) path.  No search along a path will be performed.  */
   ldin = fopen (fname, "r");
   if (ldin == NULL)
-    error (EXIT_FAILURE, errno, gettext ("cannot read version script \"%s\""),
+    error (EXIT_FAILURE, errno, gettext ("cannot read version script '%s'"),
 	   fname);
   /* No need for locking.  */
   __fsetlocking (ldin, FSETLOCKING_BYCALLER);
@@ -1500,6 +1519,8 @@ create_special_section_symbol (struct symbol **symp, const char *name)
     abort ();
 
   (*symp)->defined = 1;
+  (*symp)->local = 1;
+  (*symp)->hidden = 1;
   (*symp)->type = STT_OBJECT;
 
   ++ld_state.nsymtab;
